@@ -331,7 +331,7 @@ int mosquitto_main_loop(struct mosquitto_db *db, mosq_sock_t *listensock, int li
 #ifdef WITH_PERSISTENCE
 		if(db->config->persistence && db->config->autosave_interval){
 			if(db->config->autosave_on_changes){
-				if(db->persistence_changes > db->config->autosave_interval){
+				if(db->persistence_changes >= db->config->autosave_interval){
 					mqtt3_db_backup(db, false);
 					db->persistence_changes = 0;
 				}
@@ -498,10 +498,12 @@ static void loop_handle_reads_writes(struct mosquitto_db *db, struct pollfd *pol
 #else
 		if(pollfds[context->pollfd_index].revents & POLLIN){
 #endif
-			if(_mosquitto_packet_read(db, context)){
-				do_disconnect(db, context);
-				continue;
-			}
+			do{
+				if(_mosquitto_packet_read(db, context)){
+					do_disconnect(db, context);
+					continue;
+				}
+			}while(SSL_DATA_PENDING(context));
 		}
 	}
 }
