@@ -138,6 +138,7 @@ struct _mosquitto_subleaf {
 };
 
 struct _mosquitto_subhier {
+	struct _mosquitto_subhier *parent;
 	struct _mosquitto_subhier *children;
 	struct _mosquitto_subhier *next;
 	struct _mosquitto_subleaf *subs;
@@ -306,6 +307,7 @@ struct _mqtt3_bridge{
 	int threshold;
 	bool lazy_reconnect;
 	bool attempt_unsubscribe;
+	bool initial_notification_done;
 #ifdef WITH_TLS
 	char *tls_cafile;
 	char *tls_capath;
