@@ -25,18 +25,14 @@ pubrec_packet = mosq_test.gen_pubrec(mid)
 pubrel_packet = mosq_test.gen_pubrel(mid)
 pubcomp_packet = mosq_test.gen_pubcomp(mid)
 
-sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-sock.connect(("localhost", 1888))
-sock.send(connect_packet)
+sock = mosq_test.do_client_connect(connect_packet, connack_packet, connack_error="helper connack")
+sock.send(publish_packet)
 
-if mosq_test.expect_packet(sock, "helper connack", connack_packet):
-    sock.send(publish_packet)
+if mosq_test.expect_packet(sock, "helper pubrec", pubrec_packet):
+    sock.send(pubrel_packet)
 
-    if mosq_test.expect_packet(sock, "helper pubrec", pubrec_packet):
-        sock.send(pubrel_packet)
-
-        if mosq_test.expect_packet(sock, "helper pubcomp", pubcomp_packet):
-            rc = 0
+    if mosq_test.expect_packet(sock, "helper pubcomp", pubcomp_packet):
+        rc = 0
 
 sock.close()
 
