@@ -25,11 +25,9 @@ keepalive = 10
 connect_packet = mosq_test.gen_connect("connect-success-test", keepalive=keepalive)
 connack_packet = mosq_test.gen_connack(rc=0)
 
-broker = subprocess.Popen(['../../src/mosquitto', '-c', '08-ssl-connect-no-auth.conf'], stderr=subprocess.PIPE)
+broker = mosq_test.start_broker(filename=os.path.basename(__file__), port=1889)
 
 try:
-    time.sleep(0.5)
-
     sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
     ssock = ssl.wrap_socket(sock, ca_certs="../ssl/test-root-ca.crt", cert_reqs=ssl.CERT_REQUIRED, ssl_version=ssl.PROTOCOL_TLSv1)
     ssock.settimeout(20)
